@@ -248,6 +248,7 @@ adw_preferences_group_class_init (AdwPreferencesGroupClass *klass)
   object_class->dispose = adw_preferences_group_dispose;
 
   widget_class->compute_expand = adw_widget_compute_expand;
+  widget_class->focus = adw_widget_focus_child;
 
   /**
    * AdwPreferencesGroup:title: (attributes org.gtk.Property.get=adw_preferences_group_get_title org.gtk.Property.set=adw_preferences_group_set_title)
@@ -374,6 +375,7 @@ adw_preferences_group_add (AdwPreferencesGroup *self,
 
   g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
   g_return_if_fail (GTK_IS_WIDGET (child));
+  g_return_if_fail (gtk_widget_get_parent (child) == NULL);
 
   priv = adw_preferences_group_get_instance_private (self);
 
@@ -507,7 +509,7 @@ adw_preferences_group_set_description (AdwPreferencesGroup *self,
 }
 
 /**
- * adw_preferences_group_get_header_suffix:
+ * adw_preferences_group_get_header_suffix: (attributes org.gtk.Method.get_property=header-suffix)
  * @self: a `AdwPreferencesGroup`
  *
  * Gets the suffix for @self's header.
@@ -529,7 +531,7 @@ adw_preferences_group_get_header_suffix (AdwPreferencesGroup *self)
 }
 
 /**
- * adw_preferences_group_set_header_suffix:
+ * adw_preferences_group_set_header_suffix: (attributes org.gtk.Method.set_property=header-suffix)
  * @self: a `AdwPreferencesGroup`
  * @suffix: (nullable): the suffix to set
  *
@@ -550,6 +552,9 @@ adw_preferences_group_set_header_suffix (AdwPreferencesGroup *self,
   g_return_if_fail (ADW_IS_PREFERENCES_GROUP (self));
   g_return_if_fail (suffix == NULL || GTK_IS_WIDGET (suffix));
 
+  if (suffix)
+    g_return_if_fail (gtk_widget_get_parent (suffix) == NULL);
+
   priv = adw_preferences_group_get_instance_private (self);
 
   if (suffix == priv->header_suffix)
@@ -564,6 +569,8 @@ adw_preferences_group_set_header_suffix (AdwPreferencesGroup *self,
     gtk_box_append (priv->header_box, priv->header_suffix);
 
   update_header_visibility (self);
+
+  g_object_notify_by_pspec (G_OBJECT (self), props[PROP_HEADER_SUFFIX]);
 }
 
 /**
